﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.BioBoost.ProblemDescription;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Random;
using System;
using System.Linq;

namespace HeuristicLab.BioBoost.Operators.Mutation {
  [StorableClass]
  public class CommonTargetIntegerVectorMutator : BoundedIntegerVectorManipulator {

    public LookupParameter<BioBoostProblemData> ProblemDataParameter {
      get { return (LookupParameter<BioBoostProblemData>) Parameters["ProblemData"]; }
    }

    public ValueLookupParameter<DoubleValue> AvgBundleMutationRatioParameter {
      get { return (ValueLookupParameter<DoubleValue>) Parameters["AvgBundleMutationRatio"]; }
    }

    public double AvgBundleMutationRatio {
      get { return AvgBundleMutationRatioParameter.ActualValue.Value; }
    }


    public BioBoostProblemData ProblemData {
      get { return ProblemDataParameter.ActualValue; }
    }

    #region Construction & Cloning
    [StorableConstructor]
    protected CommonTargetIntegerVectorMutator(bool isDeserializing) : base(isDeserializing) {}
    protected CommonTargetIntegerVectorMutator(CommonTargetIntegerVectorMutator orig, Cloner cloner) : base(orig, cloner) {}
    public CommonTargetIntegerVectorMutator() {
      Parameters.Add(new LookupParameter<BioBoostProblemData>("ProblemData", "The problem instance description container."));
      Parameters.Add(new ValueLookupParameter<DoubleValue>("AvgBundleMutationRatio", "The average ratio of modified targets in a bundle.", new PercentValue(1)));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new CommonTargetIntegerVectorMutator(this, cloner);
    }
    #endregion

    protected override void ManipulateBounded(IRandom random, IntegerVector integerVector, IntMatrix bounds) {
      var ratio = new NormalDistributedRandom(random, Math.Min(1, Math.Max(0, AvgBundleMutationRatio)), 0.1).NextDouble();
      var targetGroups = integerVector.Select((t, i) => new {t, i}).GroupBy(x => x.t, x => x.i).ToList();
      var targetGroup = targetGroups[random.Next(targetGroups.Count)];
      var targets = targetGroup.Shuffle(random).ToList();
      var newTarget = ProblemData.Neighbors.GetRandomNeighbor(targetGroup.Key, random, bounds[0, 1]);
      foreach (var i in targets.Take(Math.Max(1, (int)Math.Round(ratio*(targets.Count-1))))) {
        integerVector[i] = newTarget;
      }
    }
  }
}
